@@ -10,11 +10,11 @@ try:
     from importlib import metadata
 except ImportError:  # pragma: no cover
     import importlib_metadata as metadata  # type: ignore
+
 PACKAGE_VERSION = metadata.version("pulse3D")
 
 TWENTY_FOUR_WELL_PLATE = LabwareDefinition(row_count=4, column_count=6)
 
-
 NUM_CHANNELS_24_WELL_PLATE = 24 * NUM_CHANNELS_PER_WELL
 
 MIN_SUPPORTED_FILE_VERSION = "0.1.1"
@@ -23,6 +23,8 @@ CURRENT_BETA2_HDF5_FILE_FORMAT_VERSION = "1.0.0"
 FILE_FORMAT_VERSION_METADATA_KEY = "File Format Version"
 FILE_MIGRATION_PATHS = immutabledict({"0.3.1": "0.4.1", "0.4.1": "0.4.2"})
 
+NOT_APPLICABLE_LABEL = "N/A"
+
 # Eli (1/19/21): H5 files can't store the concept of `None` in their metadata, so using this value to denote that a particular piece of metadata is not available (i.e. after migrating to a newer file format version)
 NOT_APPLICABLE_H5_METADATA = uuid.UUID("59d92e00-99d5-4460-9a28-5a1a0fe9aecf")
 
@@ -77,6 +79,8 @@ IS_CALIBRATION_FILE_UUID = uuid.UUID("9a6f90eb-fe34-423b-bfed-fb441d6d9e5f")
 CHANNEL_FIRMWARE_VERSION_UUID = uuid.UUID("d9694cfe-824c-41f8-915e-91e41ce7af32")
 BOOT_FLAGS_UUID = uuid.UUID("762f6715-ffcd-4e8d-b707-638dd5777841")
 INITIAL_MAGNET_FINDING_PARAMS_UUID = uuid.UUID("da5f2f6d-6874-4e53-be10-90c4bfbd3d45")
+PLATEMAP_NAME_UUID = uuid.UUID("2663ebdb-dce8-4fa1-8ec6-383db1ed1dbb")
+PLATEMAP_LABEL_UUID = uuid.UUID("f4c1a517-4c0b-4970-9253-a5b4aa64362f")
 METADATA_UUID_DESCRIPTIONS = immutabledict(
     {
         # General values
@@ -115,6 +119,8 @@ METADATA_UUID_DESCRIPTIONS = immutabledict(
         ORIGINAL_FILE_VERSION_UUID: "The original version of the file when recorded, prior to any migrations to newer versions/formats.",
         UTC_TIMESTAMP_OF_FILE_VERSION_MIGRATION_UUID: "Timestamp when this file was migrated from an earlier version.",
         FILE_VERSION_PRIOR_TO_MIGRATION_UUID: "File format version that this file was migrated from",
+        PLATEMAP_NAME_UUID: "The name of the Platemap Well Grouping",
+        PLATEMAP_LABEL_UUID: "The name of a label in the Platemap Well Grouping",
         # Beta 1 specific values
         XEM_SERIAL_NUMBER_UUID: "XEM Serial Number",
         # Beta 2 specific values
@@ -138,6 +144,7 @@ METADATA_UUID_DESCRIPTIONS = immutabledict(
 )
 
 DATETIME_STR_FORMAT = "%Y-%m-%d %H:%M:%S.%f"
+
 MILLI_TO_BASE_CONVERSION = int(1e3)
 CENTIMILLISECONDS_PER_SECOND = int(1e5)
 MICRO_TO_BASE_CONVERSION = int(1e6)
@@ -193,11 +200,35 @@ DEFAULT_TWITCH_WIDTH_PERCENTS = tuple(range(10, 95, 5))
 DEFAULT_PROMINENCE_FACTORS = (6, 6)
 DEFAULT_WIDTH_FACTORS = (7, 7)
 
+DEFAULT_NB_NOISE_PROMINENCE_FACTOR = 2.5
+DEFAULT_NB_RELATIVE_PROMINENCE_FACTOR = 0.2
+DEFAULT_NB_WIDTH_FACTORS = (0, 5)
+DEFAULT_NB_HEIGHT_FACTOR = 0
+DEFAULT_NB_VALLEY_SEARCH_DUR = 1
+DEFAULT_NB_UPSLOPE_DUR = 0.07
+DEFAULT_NB_UPSLOPE_NOISE_ALLOWANCE_DUR = 0.01
+DEFAULT_MAX_FREQUENCY = None
+
+DEFAULT_NB_PARAMS = immutabledict(
+    {
+        DEFAULT_NB_NOISE_PROMINENCE_FACTOR: "Noise Prominence Factor",
+        DEFAULT_NB_RELATIVE_PROMINENCE_FACTOR: "Relative Prominence Factor",
+        DEFAULT_NB_WIDTH_FACTORS: "Width Factor",
+        DEFAULT_NB_HEIGHT_FACTOR: "Min Peak Height",
+        DEFAULT_NB_VALLEY_SEARCH_DUR: "Valley Search Duration",
+        DEFAULT_NB_UPSLOPE_DUR: "Upslope Duration",
+        DEFAULT_NB_UPSLOPE_NOISE_ALLOWANCE_DUR: "Upslope Noise Allowance Duration",
+        DEFAULT_MAX_FREQUENCY: "Max Frequency of Peaks",
+    }
+)
+
+# twitch indices keys
 PRIOR_PEAK_INDEX_UUID = uuid.UUID("80df90dc-21f8-4cad-a164-89436909b30a")
 PRIOR_VALLEY_INDEX_UUID = uuid.UUID("72ba9466-c203-41b6-ac30-337b4a17a124")
 SUBSEQUENT_PEAK_INDEX_UUID = uuid.UUID("7e37325b-6681-4623-b192-39f154350f36")
 SUBSEQUENT_VALLEY_INDEX_UUID = uuid.UUID("fd47ba6b-ee4d-4674-9a89-56e0db7f3d97")
 
+# filters
 BESSEL_BANDPASS_UUID = uuid.UUID("0ecf0e52-0a29-453f-a6ff-46f5ec3ae783")
 BESSEL_LOWPASS_10_UUID = uuid.UUID("7d64cac3-b841-4912-b734-c0cf20a81e7a")
 BESSEL_LOWPASS_30_UUID = uuid.UUID("eee66c75-4dc4-4eb4-8d48-6c608bf28d91")
@@ -237,6 +268,7 @@ MIN_EXPERIMENT_ID = 0
 MAX_CARDIAC_EXPERIMENT_ID = 99
 MAX_SKM_EXPERIMENT_ID = 199
 MAX_VARIABLE_EXPERIMENT_ID = 299
+MAX_MINI_EXPERIMENT_ID = 399
 MAX_EXPERIMENT_ID = 999
 
 CARDIAC_STIFFNESS_LABEL = "Cardiac (1x)"
@@ -279,7 +311,7 @@ DEFAULT_CELL_HEIGHT = 20
 CHART_ALPHA = 60  # for full/snapshots -- num pixels between left figure edge and plot area
 CHART_GAMMA = 150  # for full/snapshots -- num pixels between right figure edge and plot area
 CHART_PIXELS_PER_SECOND = 35  # for full/snapshots -- number of pixels per second
-CHART_MAXIMUM_SNAPSHOT_LENGTH = 10
+CHART_MAXIMUM_SNAPSHOT_LENGTH_SECS = 10
 
 CHART_HEIGHT_CELLS = 15
 CHART_HEIGHT = DEFAULT_CELL_HEIGHT * CHART_HEIGHT_CELLS
@@ -290,8 +322,6 @@ STIM_CHART_HEIGHT = DEFAULT_CELL_HEIGHT * STIM_CHART_HEIGHT_CELLS
 CHART_FIXED_WIDTH_CELLS = 8
 CHART_FIXED_WIDTH = DEFAULT_CELL_WIDTH * CHART_FIXED_WIDTH_CELLS
 
-PEAK_VALLEY_COLUMN_START = 100
-STIM_DATA_COLUMN_START = 200
 SECONDS_PER_CELL = 2.5
 
 CALCULATED_METRIC_DISPLAY_NAMES = {
