@@ -7,26 +7,25 @@ If a new metric is requested, you must implement `fit`,
 
 # for hashing dataframes
 import abc
-from functools import lru_cache
-from functools import partial
-from hashlib import sha256
 from typing import Any
 from typing import Dict
-from typing import List
 from typing import Optional
 from typing import Tuple
 from typing import Union
 from uuid import UUID
 
-from nptyping import Float64
 from nptyping import NDArray
 import numpy as np
 import pandas as pd
 from pandas import DataFrame
 from pandas import Series
-from pandas.util import hash_pandas_object
+from pulse3D.transforms import get_time_window_indices
 
+from .compression_cy import interpolate_x_for_y_between_two_points
+from .compression_cy import interpolate_y_for_x_between_two_points
+from .constants import DEFAULT_BASELINE_WIDTHS
 from .constants import DEFAULT_TWITCH_WIDTH_PERCENTS
+from .constants import INTERPOLATED_DATA_PERIOD_SECONDS
 from .constants import MICRO_TO_BASE_CONVERSION
 from .constants import PRIOR_VALLEY_INDEX_UUID
 from .constants import SUBSEQUENT_VALLEY_INDEX_UUID
@@ -53,7 +52,7 @@ class BaseMetric:
         filtered_data: NDArray[(2, Any), int],
         twitch_indices: Dict[int, Dict[UUID, Optional[int]]],
         **kwargs: Dict[str, Any],
-    ) -> Union[NDArray[Float64], List[Dict[int, Dict[UUID, Any]]], DataFrame]:
+    ) -> Series:
         pass
 
     def add_per_twitch_metrics(
@@ -115,8 +114,15 @@ class BaseMetric:
 class TwitchAmplitude(BaseMetric):
     """Calculate the amplitude for each twitch."""
 
-    def __init__(self, rounded: bool = False, **kwargs: Dict[str, Any]):
+    def __init__(
+        self,
+        rounded: bool = False,
+        baseline_widths_to_use: Tuple[int, ...] = DEFAULT_BASELINE_WIDTHS,
+        **kwargs: Dict[str, Any],
+    ):
         super().__init__(rounded=rounded, **kwargs)
+        # C10 in the metric definition diagram is the C point at 90% twitch width
+        self.baseline_widths = [100 - baseline_widths_to_use[0], baseline_widths_to_use[1]]
 
     def fit(
         self,
@@ -124,10 +130,12 @@ class TwitchAmplitude(BaseMetric):
         filtered_data: NDArray[(2, Any), int],
         twitch_indices: Dict[int, Dict[UUID, Optional[int]]],
         **kwargs: Dict[str, Any],
-    ) -> DataFrame:
-
+    ) -> Series:
         amplitudes = self.calculate_amplitudes(
-            twitch_indices=twitch_indices, filtered_data=filtered_data, rounded=self.rounded
+            twitch_indices=twitch_indices,
+            filtered_data=filtered_data,
+            rounded=self.rounded,
+            baseline_widths=tuple(self.baseline_widths),
         )
 
         return amplitudes
@@ -136,6 +144,7 @@ class TwitchAmplitude(BaseMetric):
     def calculate_amplitudes(
         twitch_indices: Dict[int, Dict[UUID, Optional[int]]],
         filtered_data: NDArray[(2, Any), int],
+        baseline_widths: Tuple[int, ...],
         rounded: bool = False,
     ) -> Series:
         """Get the amplitudes for all twitches.
@@ -144,40 +153,37 @@ class TwitchAmplitude(BaseMetric):
         is calculated as the mean distance from peak to both valleys.
 
         Args:
-            twitch_indices: a dictionary in which the key is an integer representing the time points
-                of all the peaks of interest and the value is an inner dictionary with various UUID
+            twitch_indices: a dictionary in which the key is an integer representing (TODO fix this for every metric) of interest and the value is an inner dictionary with various UUID
                 of prior/subsequent peaks and valleys and their index values.
 
             filtered_data: a 2D array of the time and value (magnetic, voltage, displacement, force)
                 data after it has gone through noise filtering
 
+            baseline_widths: tuple twitch widths to use as baseline metrics
+
         Returns:
             Pandas Series of float values representing the amplitude of each twitch
         """
-        twitch_width = 90
-
-        twitch_indices_hashable = HashableDataFrame(DataFrame.from_dict(twitch_indices).T)
-        filtered_data_hashable = tuple(tuple(d) for d in filtered_data)
         _, coordinates = TwitchWidth.calculate_twitch_widths(
-            twitch_indices=twitch_indices_hashable,
-            filtered_data=filtered_data_hashable,
+            filtered_data=filtered_data,
+            twitch_indices=twitch_indices,
+            twitch_width_percents=baseline_widths,
             rounded=rounded,
-            twitch_width_percents=(twitch_width,),
+            as_dict=True,  # Tanner (1/20/23): using as_dict=True here to speed this up. This calculation is performed in the MA Controller, so it must be as fast as possible
         )
 
         estimates_dict: Dict[int, float] = dict()
 
-        for twitch_peak_idx, twitch_data in coordinates.iterrows():
+        for twitch_peak_idx, twitch_data in coordinates.items():
             twitch_peak_x, twitch_peak_y = filtered_data[:, twitch_peak_idx]
 
-            # C10 in the metric definition diagram is the C point at 90% twitch width
-            c10x = twitch_data.loc["time", "contraction", twitch_width]
-            c10y = twitch_data.loc["force", "contraction", twitch_width]
-            r90x = twitch_data.loc["time", "relaxation", twitch_width]
-            r90y = twitch_data.loc["force", "relaxation", twitch_width]
+            c10x = twitch_data["time"]["contraction"][baseline_widths[0]]
+            c10y = twitch_data["force"]["contraction"][baseline_widths[0]]
+            r90x = twitch_data["time"]["relaxation"][baseline_widths[1]]
+            r90y = twitch_data["force"]["relaxation"][baseline_widths[1]]
 
             twitch_base_y = interpolate_y_for_x_between_two_points(twitch_peak_x, c10x, c10y, r90x, r90y)
-            amplitude_y = (twitch_peak_y - twitch_base_y) * MICRO_TO_BASE_CONVERSION
+            amplitude_y = twitch_peak_y - twitch_base_y
 
             estimates_dict[twitch_peak_idx] = amplitude_y
 
@@ -188,8 +194,10 @@ class TwitchAmplitude(BaseMetric):
 class TwitchFractionAmplitude(TwitchAmplitude):
     """Calculate the fraction of max amplitude for each twitch."""
 
-    def __init__(self, **kwargs: Dict[str, Any]):
-        super().__init__(rounded=False, **kwargs)
+    def __init__(
+        self, baseline_widths_to_use: Tuple[int, ...] = DEFAULT_BASELINE_WIDTHS, **kwargs: Dict[str, Any]
+    ):
+        super().__init__(rounded=False, baseline_widths_to_use=baseline_widths_to_use, **kwargs)
 
     def fit(
         self,
@@ -197,7 +205,7 @@ class TwitchFractionAmplitude(TwitchAmplitude):
         filtered_data: NDArray[(2, Any), int],
         twitch_indices: Dict[int, Dict[UUID, Optional[int]]],
         **kwargs: Dict[str, Any],
-    ) -> DataFrame:
+    ) -> Series:
         amplitudes = super().fit(
             peak_and_valley_indices=peak_and_valley_indices,
             twitch_indices=twitch_indices,
@@ -207,39 +215,6 @@ class TwitchFractionAmplitude(TwitchAmplitude):
         return estimates
 
 
-class TwitchWidthCoordinates(BaseMetric):
-    """Calculate contraction (or relaxation) percent-width coordinates."""
-
-    def __init__(
-        self,
-        rounded: bool = False,
-        twitch_width_percents: Tuple[int, ...] = DEFAULT_TWITCH_WIDTH_PERCENTS,
-        **kwargs: Dict[str, Any],
-    ):
-        super().__init__(rounded=rounded, **kwargs)
-
-        self.twitch_width_percents = twitch_width_percents
-
-    def fit(
-        self,
-        peak_and_valley_indices: Tuple[NDArray[int], NDArray[int]],
-        filtered_data: NDArray[(2, Any), int],
-        twitch_indices: Dict[int, Dict[UUID, Optional[int]]],
-        **kwargs: Dict[str, Any],
-    ) -> DataFrame:
-        twitch_indices_hashable = HashableDataFrame(DataFrame.from_dict(twitch_indices).T)
-        filtered_data_hashable = tuple(tuple(d) for d in filtered_data)
-
-        _, coordinates = TwitchWidth.calculate_twitch_widths(
-            twitch_indices=twitch_indices_hashable,
-            filtered_data=filtered_data_hashable,
-            rounded=self.rounded,
-            twitch_width_percents=tuple(self.twitch_width_percents),
-        )
-
-        return coordinates
-
-
 class TwitchWidth(BaseMetric):
     """Calculate the width of each twitch at fraction of twitch."""
 
@@ -260,14 +235,11 @@ class TwitchWidth(BaseMetric):
         twitch_indices: Dict[int, Dict[UUID, Optional[int]]],
         **kwargs: Dict[str, Any],
     ) -> DataFrame:
-        twitch_indices_hashable = HashableDataFrame(DataFrame.from_dict(twitch_indices).T)
-        filtered_data_hashable = tuple(tuple(d) for d in filtered_data)
-
         widths, _ = self.calculate_twitch_widths(
-            twitch_indices=twitch_indices_hashable,
-            filtered_data=filtered_data_hashable,
-            rounded=self.rounded,
+            filtered_data=filtered_data,
+            twitch_indices=twitch_indices,
             twitch_width_percents=tuple(self.twitch_width_percents),
+            rounded=self.rounded,
         )
 
         return widths
@@ -279,18 +251,16 @@ class TwitchWidth(BaseMetric):
             aggregate_df[metric_id, iter_percent] = aggregate_estimates
 
     @staticmethod
-    @lru_cache()
     def calculate_twitch_widths(
-        twitch_indices: DataFrame,
-        filtered_data: Tuple[Tuple[float], Tuple[float]],
-        rounded: bool = False,
+        filtered_data: NDArray[(2, Any), int],
+        twitch_indices: Dict[int, Dict[UUID, Optional[int]]],
         twitch_width_percents: Tuple[int, ...] = DEFAULT_TWITCH_WIDTH_PERCENTS,
+        rounded: bool = False,
+        as_dict: bool = False,
     ) -> Tuple[DataFrame, DataFrame]:
         """Determine twitch width between 10-90% down to the nearby valleys.
 
-        This is a time-consuming step, so we use a hashable DataFrame for the
-        twitch indices, and a tuple of tuples for the filtered data.
-
+        TODO
         Args:
             twitch_indices: a HashableDataFrame in which index is an integer representing the time points
                 of all the peaks of interest and columns are UUIDs of prior/subsequent peaks and valleys and
@@ -303,27 +273,30 @@ class TwitchWidth(BaseMetric):
             is a percent-twitch width of all the peaks of interest
 
             coordinate_df: MultiIndex DataFrame, where each index is an integer representing the time points,
-            and each  column level corresponds to the time (X) / force(Y), contration (rising) / relaxation
+            and each  column level corresponds to the time (X) / force(Y), contraction (rising) / relaxation
             (falling), and percent-twitch width coordinates
         """
         coordinate_dict: Dict[int, Dict[str, Dict[str, Any]]] = dict()
 
-        width_dict: Dict[int, Dict[int, Any]] = {twitch_index: {} for twitch_index in twitch_indices.index}
+        width_dict: Dict[int, Dict[int, Any]] = {twitch_index: {} for twitch_index in twitch_indices}
 
-        time_series = filtered_data[0]
-        value_series = filtered_data[1]
+        timepoints_arr = filtered_data[0]
+        force_amplitudes_arr = filtered_data[1]
 
-        for iter_twitch_peak_idx in twitch_indices.index:
-            peak_value = value_series[iter_twitch_peak_idx]
+        twitch_width_percents = sorted(twitch_width_percents)  # type: ignore
 
-            prior_valley_idx = int(twitch_indices[PRIOR_VALLEY_INDEX_UUID][iter_twitch_peak_idx])
-            prior_valley_value = value_series[prior_valley_idx]
+        for iter_twitch_peak_idx in twitch_indices:
+            peak_force = force_amplitudes_arr[iter_twitch_peak_idx]
 
-            subsequent_valley_idx = int(twitch_indices[SUBSEQUENT_VALLEY_INDEX_UUID][iter_twitch_peak_idx])
-            subsequent_valley_value = value_series[subsequent_valley_idx]
+            # calculate magnitude of rise
+            prior_valley_idx = twitch_indices[iter_twitch_peak_idx][PRIOR_VALLEY_INDEX_UUID]  # type: ignore
+            prior_valley_force = force_amplitudes_arr[prior_valley_idx]
+            magnitude_of_rise = peak_force - prior_valley_force
 
-            rising_amplitude = peak_value - prior_valley_value
-            falling_amplitude = peak_value - subsequent_valley_value
+            # calculate magnitude of fall
+            subsequent_valley_idx = twitch_indices[iter_twitch_peak_idx][SUBSEQUENT_VALLEY_INDEX_UUID]  # type: ignore
+            subsequent_valley_force = force_amplitudes_arr[subsequent_valley_idx]
+            magnitude_of_fall = peak_force - subsequent_valley_force
 
             rising_idx = iter_twitch_peak_idx - 1
             falling_idx = iter_twitch_peak_idx + 1
@@ -333,36 +306,33 @@ class TwitchWidth(BaseMetric):
                 for metric_type in ("force", "time")
             }
 
-            for iter_percent in sorted(twitch_width_percents):
-                rising_threshold = peak_value - (iter_percent / 100) * rising_amplitude
-                falling_threshold = peak_value - (iter_percent / 100) * falling_amplitude
+            for iter_percent in twitch_width_percents:
+                rising_threshold = peak_force - (iter_percent / 100) * magnitude_of_rise
+                falling_threshold = peak_force - (iter_percent / 100) * magnitude_of_fall
 
-                # move to the left from the twitch peak until the threshold is reached
-                while abs(value_series[rising_idx] - prior_valley_value) > abs(
-                    rising_threshold - prior_valley_value
-                ):
+                # move to the left from the twitch peak until the rising threshold is reached
+                while force_amplitudes_arr[rising_idx] > rising_threshold:
                     rising_idx -= 1
                 # move to the right from the twitch peak until the falling threshold is reached
-                while abs(value_series[falling_idx] - subsequent_valley_value) > abs(
-                    falling_threshold - subsequent_valley_value
-                ):
+                while force_amplitudes_arr[falling_idx] > falling_threshold:
                     falling_idx += 1
+
                 interpolated_rising_timepoint = interpolate_x_for_y_between_two_points(
                     rising_threshold,
-                    time_series[rising_idx],
-                    value_series[rising_idx],
-                    time_series[rising_idx + 1],
-                    value_series[rising_idx + 1],
+                    timepoints_arr[rising_idx],
+                    force_amplitudes_arr[rising_idx],
+                    timepoints_arr[rising_idx + 1],
+                    force_amplitudes_arr[rising_idx + 1],
                 )
                 interpolated_falling_timepoint = interpolate_x_for_y_between_two_points(
                     falling_threshold,
-                    time_series[falling_idx],
-                    value_series[falling_idx],
-                    time_series[falling_idx - 1],
-                    value_series[falling_idx - 1],
+                    timepoints_arr[falling_idx],
+                    force_amplitudes_arr[falling_idx],
+                    timepoints_arr[falling_idx - 1],
+                    force_amplitudes_arr[falling_idx - 1],
                 )
-
                 width_val = interpolated_falling_timepoint - interpolated_rising_timepoint
+
                 if rounded:
                     width_val = int(round(width_val, 0))
                     interpolated_falling_timepoint = int(round(interpolated_falling_timepoint, 0))
@@ -380,13 +350,16 @@ class TwitchWidth(BaseMetric):
             # fill coordinate value dictionary
             coordinate_dict[iter_twitch_peak_idx] = twitch_dict
 
+        if as_dict:
+            return width_dict, coordinate_dict
+
         # convert coordinate dictionary to dataframe
         coordinate_df = pd.DataFrame.from_dict(
             {
                 (twitch_index, metric_type, contraction_type, twitch_width): coordinate_dict[twitch_index][
                     metric_type
                 ][contraction_type][twitch_width]
-                for twitch_index in twitch_indices.index
+                for twitch_index in twitch_indices
                 for metric_type in ("force", "time")
                 for contraction_type in ["contraction", "relaxation"]
                 for twitch_width in twitch_width_percents
@@ -414,19 +387,15 @@ class TwitchVelocity(BaseMetric):
         self,
         rounded: bool = False,
         is_contraction: bool = True,
-        twitch_width_percents: Tuple[int, ...] = DEFAULT_TWITCH_WIDTH_PERCENTS,
+        baseline_widths_to_use: Tuple[int, ...] = DEFAULT_BASELINE_WIDTHS,
         **kwargs: Dict[str, Any],
     ):
         super().__init__(rounded=rounded, **kwargs)
 
-        self.twitch_width_percents = twitch_width_percents
-
-        velocity_start = min(self.twitch_width_percents)
-        velocity_end = max(self.twitch_width_percents)
-        self.velocity_index_start = self.twitch_width_percents.index(velocity_start)
-        self.velocity_index_end = self.twitch_width_percents.index(velocity_end)
-
+        self.baseline_widths = baseline_widths_to_use
         self.is_contraction = is_contraction
+        # always need the 10 for relaxation and 90 for contraction to compare against input baseline width
+        self.twitch_widths = set(self.baseline_widths) | {10, 90}
 
     def fit(
         self,
@@ -435,28 +404,20 @@ class TwitchVelocity(BaseMetric):
         twitch_indices: Dict[int, Dict[UUID, Optional[int]]],
         **kwargs: Dict[str, Any],
     ) -> Series:
-        twitch_indices_hashable = HashableDataFrame(DataFrame.from_dict(twitch_indices).T)
-        filtered_data_hashable = tuple(tuple(d) for d in filtered_data)
-
         _, coordinates = TwitchWidth.calculate_twitch_widths(
-            twitch_indices=twitch_indices_hashable,
-            filtered_data=filtered_data_hashable,
+            filtered_data=filtered_data,
+            twitch_indices=twitch_indices,
+            twitch_width_percents=tuple(self.twitch_widths),
             rounded=self.rounded,
-            twitch_width_percents=tuple(self.twitch_width_percents),
         )
 
         velocities = self.calculate_twitch_velocity(
-            twitch_indices=twitch_indices, coordinate_df=coordinates, is_contraction=self.is_contraction
+            coordinate_df=coordinates, is_contraction=self.is_contraction
         )
 
         return velocities
 
-    def calculate_twitch_velocity(
-        self,
-        twitch_indices: Dict[int, Dict[UUID, Optional[int]]],
-        coordinate_df: DataFrame,
-        is_contraction: bool,
-    ) -> Series:
+    def calculate_twitch_velocity(self, coordinate_df: DataFrame, is_contraction: bool) -> Series:
         """Find the velocity for each twitch.
 
         Args:
@@ -476,10 +437,14 @@ class TwitchVelocity(BaseMetric):
         Returns:
             DataFrame floats that are the velocities of each twitch
         """
-        coord_type = "contraction" if is_contraction else "relaxation"
-
-        twitch_base = self.twitch_width_percents[self.velocity_index_end]
-        twitch_top = self.twitch_width_percents[self.velocity_index_start]
+        if is_contraction:
+            coord_type = "contraction"
+            twitch_base = self.baseline_widths[0]
+            twitch_top = 90
+        else:
+            coord_type = "relaxation"
+            twitch_base = self.baseline_widths[1]
+            twitch_top = 10
 
         Y_end = coordinate_df["force", coord_type, twitch_top]
         Y_start = coordinate_df["force", coord_type, twitch_base]
@@ -489,7 +454,7 @@ class TwitchVelocity(BaseMetric):
 
         # change in force / change in time
         velocity = abs((Y_end - Y_start) / (X_end - X_start))
-        velocity *= MICRO_TO_BASE_CONVERSION**2
+        velocity *= MICRO_TO_BASE_CONVERSION
 
         return velocity
 
@@ -498,7 +463,6 @@ class TwitchIrregularity(BaseMetric):
     """Calculate irregularity of each twitch."""
 
     def __init__(self, rounded: bool = False, **kwargs: Dict[str, Any]):
-
         super().__init__(rounded=rounded, **kwargs)
 
     def fit(
@@ -519,7 +483,6 @@ class TwitchIrregularity(BaseMetric):
     ) -> None:
         statistics_dict = self.create_statistics_df(metric=metrics[1:-1], rounded=self.rounded)
         statistics_dict["n"] += 2
-
         aggregate_dict[metric_id] = statistics_dict
 
     @staticmethod
@@ -565,13 +528,12 @@ class TwitchAUC(BaseMetric):
     def __init__(
         self,
         rounded: bool = False,
-        twitch_width_percents: Tuple[int, ...] = DEFAULT_TWITCH_WIDTH_PERCENTS,
+        baseline_widths_to_use: Tuple[int, ...] = DEFAULT_BASELINE_WIDTHS,
         **kwargs: Dict[str, Any],
     ):
-
         super().__init__(rounded=rounded, **kwargs)
-
-        self.twitch_width_percents = twitch_width_percents
+        # C10 in the metric definition diagram is the C point at 90% twitch width
+        self.baseline_widths = [100 - baseline_widths_to_use[0], baseline_widths_to_use[1]]
 
     def fit(
         self,
@@ -580,18 +542,18 @@ class TwitchAUC(BaseMetric):
         twitch_indices: Dict[int, Dict[UUID, Optional[int]]],
         **kwargs: Dict[str, Any],
     ) -> Series:
-        twitch_indices_hashable = HashableDataFrame(DataFrame.from_dict(twitch_indices).T)
-        filtered_data_hashable = tuple(tuple(d) for d in filtered_data)
-
         _, coordinates = TwitchWidth.calculate_twitch_widths(
-            twitch_indices=twitch_indices_hashable,
-            filtered_data=filtered_data_hashable,
+            filtered_data=filtered_data,
+            twitch_indices=twitch_indices,
+            twitch_width_percents=tuple(self.baseline_widths),
             rounded=self.rounded,
-            twitch_width_percents=tuple(self.twitch_width_percents),
         )
 
         auc = self.calculate_area_under_curve(
-            twitch_indices=twitch_indices, filtered_data=filtered_data, coordinate_df=coordinates
+            twitch_indices=twitch_indices,
+            filtered_data=filtered_data,
+            coordinate_df=coordinates,
+            baseline_widths=tuple(self.baseline_widths),
         )
         return auc
 
@@ -600,6 +562,7 @@ class TwitchAUC(BaseMetric):
         twitch_indices: Dict[int, Dict[UUID, Optional[int]]],
         filtered_data: NDArray[(2, Any), int],
         coordinate_df: DataFrame,
+        baseline_widths: Tuple[int, ...],
     ) -> Series:
         """Calculate the area under the curve (AUC) for twitches.
 
@@ -609,96 +572,30 @@ class TwitchAUC(BaseMetric):
                 of prior/subsequent peaks and valleys and their index values.
 
             filtered_data: a 2D array of the time and value (magnetic, voltage, displacement, force)
-                data after it has gone through noise filtering
+                data after it has gone through noise filtering and interpolation
 
             per_twitch_widths: a list of dictionaries where the first key is the percentage of the
                 way down to the nearby valleys, the second key is a UUID representing either the
                 value of the width, or the rising or falling coordinates. The final value is either
                 an int representing the width value or a tuple of ints for the x/y coordinates
 
+            baseline_widths: tuple twitch widths to use as baseline metrics
+
         Returns:
             Pandas Series of floats representing area under the curve for each twitch
         """
-        width_percent = 90  # what percent of repolarization to use as the bottom limit for calculating AUC
         estimates_dict: Dict[int, Union[int, float]] = dict()
 
-        value_series = filtered_data[1, :]
-        time_series = filtered_data[0, :]
-
         rising_x_values = coordinate_df["time"]["contraction"].T.to_dict()
-        rising_y_values = coordinate_df["force"]["contraction"].T.to_dict()
-
         falling_x_values = coordinate_df["time"]["relaxation"].T.to_dict()
-        falling_y_values = coordinate_df["force"]["relaxation"].T.to_dict()
 
-        for iter_twitch_peak_idx, iter_twitch_indices_info in twitch_indices.items():
-            # iter_twitch_peak_timepoint = time_series[iter_twitch_peak_idx]
-            prior_valley_value = value_series[iter_twitch_indices_info[PRIOR_VALLEY_INDEX_UUID]]
-            subsequent_valley_value = value_series[iter_twitch_indices_info[SUBSEQUENT_VALLEY_INDEX_UUID]]
+        for iter_twitch_peak_idx in twitch_indices.keys():
+            start_timepoint = rising_x_values[iter_twitch_peak_idx][baseline_widths[0]]
+            stop_timepoint = falling_x_values[iter_twitch_peak_idx][baseline_widths[1]]
 
-            rising_x = rising_x_values[iter_twitch_peak_idx][width_percent]
-            rising_y = rising_y_values[iter_twitch_peak_idx][width_percent]
-            rising_coords = (rising_x, rising_y)
+            auc_window_indices = get_time_window_indices(filtered_data[0], start_timepoint, stop_timepoint)
+            auc_total = np.trapz(filtered_data[1, auc_window_indices], dx=INTERPOLATED_DATA_PERIOD_SECONDS)
 
-            falling_x = falling_x_values[iter_twitch_peak_idx][width_percent]
-            falling_y = falling_y_values[iter_twitch_peak_idx][width_percent]
-            falling_coords = (falling_x, falling_y)
-
-            auc_total: Union[float, int] = 0
-
-            # calculate area of rising side
-            rising_idx = iter_twitch_peak_idx
-            # move to the left from the twitch peak until the threshold is reached
-            while abs(value_series[rising_idx - 1] - prior_valley_value) > abs(rising_y - prior_valley_value):
-                left_x = time_series[rising_idx - 1]
-                right_x = time_series[rising_idx]
-                left_y = value_series[rising_idx - 1]
-                right_y = value_series[rising_idx]
-
-                auc_total += self.calculate_trapezoid_area(
-                    left_x,
-                    right_x,
-                    left_y,
-                    right_y,
-                    rising_coords,
-                    falling_coords,
-                )
-                rising_idx -= 1
-            # final trapezoid at the boundary of the interpolated twitch width point
-            left_x = rising_x
-            right_x = time_series[rising_idx]
-            left_y = rising_y
-            right_y = value_series[rising_idx]
-
-            auc_total += self.calculate_trapezoid_area(
-                left_x, right_x, left_y, right_y, rising_coords, falling_coords
-            )
-
-            # calculate area of falling side
-            falling_idx = iter_twitch_peak_idx
-            # move to the left from the twitch peak until the threshold is reached
-            while abs(value_series[falling_idx + 1] - subsequent_valley_value) > abs(
-                falling_y - subsequent_valley_value
-            ):
-                left_x = time_series[falling_idx]
-                right_x = time_series[falling_idx + 1]
-                left_y = value_series[falling_idx]
-                right_y = value_series[falling_idx + 1]
-
-                auc_total += self.calculate_trapezoid_area(
-                    left_x, right_x, left_y, right_y, rising_coords, falling_coords
-                )
-                falling_idx += 1
-
-            # final trapezoid at the boundary of the interpolated twitch width point
-            left_x = time_series[falling_idx]
-            right_x = falling_x
-            left_y = value_series[rising_idx]
-            right_y = falling_y
-
-            auc_total += self.calculate_trapezoid_area(
-                left_x, right_x, left_y, right_y, rising_coords, falling_coords
-            )
             if self.rounded:
                 auc_total = int(round(auc_total, 0))
 
@@ -707,34 +604,6 @@ class TwitchAUC(BaseMetric):
         estimates = pd.Series(estimates_dict)
         return estimates
 
-    @staticmethod
-    def calculate_trapezoid_area(
-        left_x: int,
-        right_x: int,
-        left_y: int,
-        right_y: int,
-        rising_coords: Tuple[Union[float, int], Union[float, int]],
-        falling_coords: Tuple[Union[float, int], Union[float, int]],
-    ) -> Union[int, float]:
-        """Calculate the area under the trapezoid.
-
-        Returns: area of the trapezoid
-        """
-        rising_x, rising_y = rising_coords
-        falling_x, falling_y = falling_coords
-
-        interp_y_for_lower_bound = partial(
-            interpolate_y_for_x_between_two_points, x_1=rising_x, y_1=rising_y, x_2=falling_x, y_2=falling_y
-        )
-
-        trapezoid_left_side = abs(left_y - interp_y_for_lower_bound(left_x))
-        trapezoid_right_side = abs(right_y - interp_y_for_lower_bound(right_x))
-
-        trapezoid_h = right_x - left_x
-        auc_total = (trapezoid_left_side + trapezoid_right_side) / 2 * trapezoid_h
-
-        return auc_total
-
 
 class TwitchPeriod(BaseMetric):
     """Calculate period of each twitch."""
@@ -748,7 +617,7 @@ class TwitchPeriod(BaseMetric):
         filtered_data: NDArray[(2, Any), int],
         twitch_indices: Dict[int, Dict[UUID, Optional[int]]],
         **kwargs: Dict[str, Any],
-    ) -> DataFrame:
+    ) -> Series:
         periods = self.calculate_twitch_period(
             twitch_indices=twitch_indices,
             peak_indices=peak_and_valley_indices[0],
@@ -837,14 +706,11 @@ class TwitchPeakTime(BaseMetric):
         twitch_indices: Dict[int, Dict[UUID, Optional[int]]],
         **kwargs: Dict[str, Any],
     ) -> DataFrame:
-        twitch_indices_hashable = HashableDataFrame(DataFrame.from_dict(twitch_indices).T)
-        filtered_data_hashable = tuple(tuple(d) for d in filtered_data)
-
         _, coordinates = TwitchWidth.calculate_twitch_widths(
-            twitch_indices=twitch_indices_hashable,
-            filtered_data=filtered_data_hashable,
-            rounded=self.rounded,
+            filtered_data=filtered_data,
+            twitch_indices=twitch_indices,
             twitch_width_percents=tuple(self.twitch_width_percents),
+            rounded=self.rounded,
         )
 
         time_difference = self.calculate_twitch_time_diff(
@@ -853,12 +719,7 @@ class TwitchPeakTime(BaseMetric):
 
         return time_difference
 
-    def add_aggregate_metrics(
-        self,
-        aggregate_df: DataFrame,
-        metric_id: UUID,
-        metrics: DataFrame,
-    ) -> None:
+    def add_aggregate_metrics(self, aggregate_df: DataFrame, metric_id: UUID, metrics: DataFrame) -> None:
         for iter_percent in self.twitch_width_percents:
             estimates = metrics[iter_percent]
             aggregate_estimates = self.create_statistics_df(estimates, rounded=self.rounded)
@@ -917,7 +778,6 @@ class TwitchPeakTime(BaseMetric):
 
         estimates_dict = {twitch_index: {} for twitch_index in twitch_indices.keys()}  # type: ignore
         for iter_twitch_idx in twitch_indices.keys():
-
             for iter_percent in self.twitch_width_percents:
                 percent = iter_percent
                 if is_contraction:
@@ -935,12 +795,7 @@ class TwitchPeakTime(BaseMetric):
 class TwitchPeakToBaseline(BaseMetric):
     """Calculate full contraction or full relaxation time."""
 
-    def __init__(
-        self,
-        rounded: bool = False,
-        is_contraction: bool = True,
-        **kwargs: Dict[str, Any],
-    ):
+    def __init__(self, rounded: bool = False, is_contraction: bool = True, **kwargs: Dict[str, Any]):
         super().__init__(rounded=rounded, **kwargs)
         self.is_contraction = is_contraction
 
@@ -968,54 +823,25 @@ class TwitchPeakToBaseline(BaseMetric):
         return estimates
 
 
-def interpolate_x_for_y_between_two_points(
-    desired_y: Union[int, float],
-    x_1: Union[int, float],
-    y_1: Union[int, float],
-    x_2: Union[int, float],
-    y_2: Union[int, float],
-) -> Union[int, float]:
-    """Find a value of x between two points that matches the desired y value.
+class WellGroupMetric(BaseMetric):
+    """Calculate aggregrate group metrics."""
 
-    Uses linear interpolation, based on point-slope formula.
-    """
-    denominator = x_2 - x_1
+    def __init__(self, **kwargs: Dict[str, Any]):
+        super().__init__(False, **kwargs)
 
-    if denominator == 0:
-        raise ZeroDivisionError("Denominator cannot be 0.")
+    def add_group_aggregate_metrics(
+        self, aggregate_df: DataFrame, metric_column: Tuple, metrics: pd.Series, metric_type
+    ) -> None:
+        """Get aggregate metrics for entire well group.
 
-    slope = (y_2 - y_1) / denominator
-    return (desired_y - y_1) / slope + x_1
-
-
-def interpolate_y_for_x_between_two_points(
-    desired_x: Union[int, float],
-    x_1: Union[int, float],
-    y_1: Union[int, float],
-    x_2: Union[int, float],
-    y_2: Union[int, float],
-) -> Union[int, float]:
-    """Find a value of y between two points that matches the desired x value.
-
-    Uses linear interpolation, based on point-slope formula.
-    """
-    denominator = x_2 - x_1
-
-    if denominator == 0:
-        raise ZeroDivisionError("Denominator cannot be 0.")
-
-    slope = (y_2 - y_1) / denominator
-    return slope * (desired_x - x_1) + y_1
-
-
-class HashableDataFrame(DataFrame):
-    def __init__(self, obj):
-        super().__init__(obj)
-
-    def __hash__(self):
-        hash_value = sha256(hash_pandas_object(self, index=True).values)
-        hash_value = hash(hash_value.hexdigest())
-        return hash_value
-
-    def __eq__(self, other):
-        return self.equals(other)
+        Args:
+            aggregate_df (DataFrame): DataFrame storing aggregate metrics for well group
+            metric_column (UUID, str): multi-index column in aggregate metrics dataframe
+            metrics (Union[NDArray[int], NDArray[float]]): estimates from all wells in single group
+            metric_type (str): scalar or by_width
+        """
+        aggregate_metrics = self.create_statistics_df(metrics.values, rounded=self.rounded)
+        if metric_type == "scalar":
+            aggregate_df[metric_column[0]] = aggregate_metrics
+        else:
+            aggregate_df[metric_column[0], metric_column[1]] = aggregate_metrics
