@@ -33,10 +33,10 @@ def peak_detector(
     """Locates peaks and valleys and returns the indices.
 
     Args:
-        filtered_magnetic_signal: a 2D array of the magnetic signal vs time data after it has gone through noise cancellation. It is assumed that the time values are in microseconds
+        filtered_magnetic_signal: a 2D array of the magnetic signal vs time data after it has gone through noise cancellation and interpolation. It is assumed that the time values are in microseconds
         twitches_point_up: whether in the incoming data stream the biological twitches are pointing up (in the positive direction) or down
-        start_time (float): start time of windowed analysis, in seconds. Default value = 0 seconds.
-        end_time (float): end time of windowed analysis, in seconds.  Default value = Inf seconds.
+        start_time (float): start time of windowed analysis, in seconds. Default value = 0 seconds. Useful for when a window is not already applied to the data
+        end_time (float): end time of windowed analysis, in seconds.  Default value = Inf seconds. Useful for when a window is not already applied to the data
         prominence_factors: (int/float, int/float) scaling factors for peak/valley prominences.  Larger values make peak-finding more flexible by reducing minimum-required prominence
         width_factors: (int/float, int/float) scaling factors for peak/valley widths.  Larger values make peak-finding more flexible by reducing minimum-required width
 
@@ -47,29 +47,24 @@ def peak_detector(
     width_factors = _format_factors(width_factors)
     prominence_factors = _format_factors(prominence_factors)
 
-    time_signal: NDArray[float] = filtered_magnetic_signal[0, :]
-    magnetic_signal: NDArray[float] = filtered_magnetic_signal[1, :]
+    # apply window
+    window_indices = get_time_window_indices(filtered_magnetic_signal[0], start_time, end_time)
+    windowed_signal = filtered_magnetic_signal[:, window_indices]
 
-    max_time = time_signal[-1]
-    start_time = np.max([0, start_time])
-    end_time = np.min([end_time, max_time / MICRO_TO_BASE_CONVERSION])
-    # if provided end time is less than or equal to start time, reset  # TODO figure out how to handle this
-    if end_time <= start_time:
-        end_time = np.inf
-
-    peak_invertor_factor, valley_invertor_factor = (1, -1) if twitches_point_up else (-1, 1)
-    sampling_period_us = filtered_magnetic_signal[0, 1] - filtered_magnetic_signal[0, 0]
+    # interpolated data points are required, meaning that the time steps should all be the same, so using the first one
+    sampling_period_us = windowed_signal[0, 1] - windowed_signal[0, 0]
 
     max_possible_twitch_freq = 7
     min_required_samples_between_twitches = int(
-        round((1 / max_possible_twitch_freq) * MICRO_TO_BASE_CONVERSION / sampling_period_us, 0),
+        round((1 / max_possible_twitch_freq) * MICRO_TO_BASE_CONVERSION / sampling_period_us, 0)
     )
+
+    magnetic_signal = windowed_signal[1, :]
     # find required height of peaks
-    max_height = np.max(magnetic_signal)
-    min_height = np.min(magnetic_signal)
-    max_prominence = abs(max_height - min_height)
+    max_prominence = abs(np.max(magnetic_signal) - np.min(magnetic_signal))
 
     # find peaks and valleys
+    peak_invertor_factor, valley_invertor_factor = (1, -1) if twitches_point_up else (-1, 1)
     peak_indices, _ = signal.find_peaks(
         magnetic_signal * peak_invertor_factor,
         width=min_required_samples_between_twitches / width_factors[0],
@@ -77,19 +72,31 @@ def peak_detector(
         prominence=max_prominence / prominence_factors[0],
     )
 
-    valley_indices, properties = signal.find_peaks(
+    valley_indices, valley_properties = signal.find_peaks(
         magnetic_signal * valley_invertor_factor,
         width=min_required_samples_between_twitches / width_factors[1],
         distance=min_required_samples_between_twitches,
         prominence=max_prominence / prominence_factors[1],
     )
-    left_ips = properties["left_ips"]
-    right_ips = properties["right_ips"]
 
+    _fix_peak_finding_results(magnetic_signal, valley_indices, valley_properties)
+
+    # indices are only valid with the given window, so adjust to match original signal
+    peak_indices += window_indices[0]
+    valley_indices += window_indices[0]
+
+    return peak_indices, valley_indices
+
+
+def _fix_peak_finding_results(magnetic_signal, valley_indices, valley_properties) -> None:
     # Patches error in B6 file for when two valleys are found in a single valley. If this is true left_bases, right_bases, prominences, and raw magnetic sensor data will also be equivalent to their previous value. This if statement indicates that the valley should be disregarded if the interpolated values on left and right intersection points of a horizontal line at the an evaluation height are equivalent. This would mean that the left and right sides of the peak and its neighbor peak align, indicating that it just one peak rather than two.
     # https://docs.scipy.org/doc/scipy/reference/generated/scipy.signal.peak_widths.html#scipy.signal.peak_widths
 
     # Tanner (10/28/21): be careful modifying any of this while loop, it is currently not unit tested
+
+    left_ips = valley_properties["left_ips"]
+    right_ips = valley_properties["right_ips"]
+
     i = 1
     while i < len(valley_indices):
         if left_ips[i] == left_ips[i - 1] and right_ips[i] == right_ips[i - 1]:  # pragma: no cover
@@ -107,20 +114,6 @@ def peak_detector(
         else:
             i += 1
 
-    # don't perform windowing of twitches unless requested
-    if start_time > 0 or end_time < max_time:
-        peak_times = time_signal[peak_indices] / MICRO_TO_BASE_CONVERSION
-        # identify peaks within time window
-        filtered_peaks = np.where((peak_times >= start_time) & (peak_times <= end_time))[0]
-        peak_indices = peak_indices[filtered_peaks]
-
-        valley_times = time_signal[valley_indices] / MICRO_TO_BASE_CONVERSION
-        # identify valleys within time window
-        filtered_valleys = np.where((valley_times >= start_time) & (valley_times <= end_time))[0]
-        valley_indices = valley_indices[filtered_valleys]
-
-    return peak_indices, valley_indices
-
 
 def _format_factors(factors):
     if isinstance(factors, int):
@@ -131,7 +124,7 @@ def _format_factors(factors):
 
 
 def find_twitch_indices(
-    peak_and_valley_indices: Tuple[NDArray[int], NDArray[int]],
+    peak_and_valley_indices: Tuple[NDArray[int], NDArray[int]]
 ) -> Dict[int, Dict[UUID, Optional[int]]]:
     """Find twitches that can be analyzed.
 
@@ -183,17 +176,16 @@ def find_twitch_indices(
         raise TwoValleysInARowError((valley_indices[valley_idx], valley_indices[valley_idx + 1]))
 
     # compile dict of twitch information
-    for itr_idx, itr_peak_index in enumerate(peak_indices):
-        if itr_idx < peak_indices.shape[0] - 1:  # last peak
-            if itr_idx == 0 and starts_with_peak:
-                continue
+    for itr_idx, itr_peak_index in enumerate(peak_indices[:-1]):
+        if itr_idx == 0 and starts_with_peak:
+            continue
 
-            twitches[itr_peak_index] = {
-                PRIOR_PEAK_INDEX_UUID: None if itr_idx == 0 else peak_indices[itr_idx - 1],
-                PRIOR_VALLEY_INDEX_UUID: valley_indices[itr_idx - 1 if starts_with_peak else itr_idx],
-                SUBSEQUENT_PEAK_INDEX_UUID: peak_indices[itr_idx + 1],
-                SUBSEQUENT_VALLEY_INDEX_UUID: valley_indices[itr_idx if starts_with_peak else itr_idx + 1],
-            }
+        twitches[itr_peak_index] = {
+            PRIOR_PEAK_INDEX_UUID: None if itr_idx == 0 else peak_indices[itr_idx - 1],
+            PRIOR_VALLEY_INDEX_UUID: valley_indices[itr_idx - 1 if starts_with_peak else itr_idx],
+            SUBSEQUENT_PEAK_INDEX_UUID: peak_indices[itr_idx + 1],
+            SUBSEQUENT_VALLEY_INDEX_UUID: valley_indices[itr_idx if starts_with_peak else itr_idx + 1],
+        }
 
     return twitches
 
@@ -233,8 +225,8 @@ def data_metrics(
 
     # Kristian (10/26/21): dictionary of metric functions. this could probably be made cleaner at some point
     metric_mapper: Dict[UUID, BaseMetric] = {
-        AMPLITUDE_UUID: TwitchAmplitude(rounded=rounded),
-        AUC_UUID: TwitchAUC(rounded=rounded, twitch_width_percents=twitch_width_percents),
+        AMPLITUDE_UUID: TwitchAmplitude(rounded=rounded, baseline_widths_to_use=baseline_widths_to_use),
+        AUC_UUID: TwitchAUC(rounded=rounded, baseline_widths_to_use=baseline_widths_to_use),
         BASELINE_TO_PEAK_UUID: TwitchPeakTime(
             rounded=rounded,
             is_contraction=True,
@@ -244,9 +236,9 @@ def data_metrics(
             rounded=rounded, is_contraction=True, twitch_width_percents=twitch_width_percents
         ),
         CONTRACTION_VELOCITY_UUID: TwitchVelocity(
-            rounded=rounded, is_contraction=True, twitch_width_percents=twitch_width_percents
+            rounded=rounded, is_contraction=True, baseline_widths_to_use=baseline_widths_to_use
         ),
-        FRACTION_MAX_UUID: TwitchFractionAmplitude(),
+        FRACTION_MAX_UUID: TwitchFractionAmplitude(baseline_widths_to_use=baseline_widths_to_use),
         IRREGULARITY_INTERVAL_UUID: TwitchIrregularity(rounded=rounded),
         PEAK_TO_BASELINE_UUID: TwitchPeakTime(
             rounded=rounded,
@@ -256,7 +248,9 @@ def data_metrics(
         RELAXATION_TIME_UUID: TwitchPeakTime(
             rounded=rounded, is_contraction=False, twitch_width_percents=twitch_width_percents
         ),
-        RELAXATION_VELOCITY_UUID: TwitchVelocity(rounded=rounded, is_contraction=False),
+        RELAXATION_VELOCITY_UUID: TwitchVelocity(
+            rounded=rounded, is_contraction=False, baseline_widths_to_use=baseline_widths_to_use
+        ),
         TWITCH_FREQUENCY_UUID: TwitchFrequency(rounded=rounded),
         TWITCH_PERIOD_UUID: TwitchPeriod(rounded=rounded),
         WIDTH_UUID: TwitchWidth(rounded=rounded, twitch_width_percents=twitch_width_percents),
@@ -423,12 +417,12 @@ def get_windowed_peaks_valleys(
         peaks: NDArray of peak indices
         valleys: NDArray of valley indices
     """
-    windowed_indices = end_idx - start_idx
+    windowed_end_idx = end_idx - start_idx - 1
     # remove up to starting index to make start time index 0
     sub_peaks = np.subtract(peaks, start_idx)
     sub_valleys = np.subtract(valleys, start_idx)
     # remove indices greater than max windowed index
-    filtered_peaks = np.where((sub_peaks < windowed_indices) & (sub_peaks >= 0))[0]
-    filtered_valleys = np.where((sub_valleys < windowed_indices) & (sub_valleys >= 0))[0]
+    peak_window_indices = get_time_window_indices(sub_peaks, 0, windowed_end_idx)
+    valley_window_indices = get_time_window_indices(sub_valleys, 0, windowed_end_idx)
     # grab indices from original sub peaks and valleys
-    return sub_peaks[filtered_peaks], sub_valleys[filtered_valleys]
+    return sub_peaks[peak_window_indices], sub_valleys[valley_window_indices]
