@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+from collections import defaultdict
 import datetime
 import glob
 import json
@@ -6,6 +7,7 @@ import logging
 import os
 import tempfile
 from typing import Any
+from typing import Dict
 from typing import List
 from typing import Optional
 from typing import Union
@@ -13,6 +15,7 @@ import uuid
 import zipfile
 
 import h5py
+from mantarray_magnet_finding.exceptions import UnableToConvergeError
 from mantarray_magnet_finding.utils import calculate_magnetic_flux_density_from_memsic
 from nptyping import NDArray
 import numpy as np
@@ -25,6 +28,8 @@ from xlsxwriter.utility import xl_cell_to_rowcol
 
 from .compression_cy import compress_filtered_magnetic_data
 from .constants import *
+from .exceptions import DuplicateWellsFoundError
+from .exceptions import IncorrectOpticalFileFormatError
 from .exceptions import NoRecordingFilesLoadedError
 from .exceptions import SubprotocolFormatIncompatibleWithInterpolationError
 from .magnet_finding import find_magnet_positions
@@ -43,8 +48,8 @@ from .transforms import create_filter
 from .transforms import noise_cancellation
 from .utils import get_experiment_id
 from .utils import get_stiffness_factor
+from .utils import get_well_name_from_h5
 from .utils import truncate
-from .utils import truncate_float
 
 log = logging.getLogger(__name__)
 
@@ -53,9 +58,7 @@ class MantarrayH5FileCreator(h5py.File):
     """Creates an H5 file with the basic format/layout."""
 
     def __init__(
-        self,
-        file_name: str,
-        file_format_version: str = CURRENT_BETA2_HDF5_FILE_FORMAT_VERSION,
+        self, file_name: str, file_format_version: str = CURRENT_BETA2_HDF5_FILE_FORMAT_VERSION
     ) -> None:
         super().__init__(
             file_name,
@@ -74,7 +77,7 @@ class WellFile:
         self,
         file_path: str,
         sampling_period: Optional[Union[int, float]] = None,
-        # TODO unit test the stiffness factor (auto and override) and has_inverted_post_magnet?
+        # TODO unit test the stiffness factor (auto and override)
         stiffness_factor: Optional[int] = None,
         has_inverted_post_magnet: bool = False,
     ):
@@ -88,79 +91,65 @@ class WellFile:
             )
 
         self.has_inverted_post_magnet = has_inverted_post_magnet
-
         self.stiffness_override = stiffness_factor is not None
 
         if file_path.endswith(".h5"):
-            with h5py.File(file_path, "r") as h5_file:
-                self.file_name = os.path.basename(h5_file.filename)
-                self.is_force_data = True
-                self.is_magnetic_data = True
+            self.is_force_data = True
+            self.is_magnetic_data = True
 
-                self.attrs = {attr: h5_file.attrs[attr] for attr in list(h5_file.attrs)}
-                self.version = self[FILE_FORMAT_VERSION_METADATA_KEY]
-                self.stimulation_protocol = self.get(STIMULATION_PROTOCOL_UUID, "null")
+            self._load_data_from_h5_file(file_path)
 
-                if self.stiffness_override:
-                    self.stiffness_factor = stiffness_factor
-                elif not self.get(IS_CALIBRATION_FILE_UUID, False):
-                    # earlier versions of files do not have the IS_CALIBRATION_FILE_UUID in their metadata
-                    experiment_id = get_experiment_id(self[PLATE_BARCODE_UUID])
-                    self.stiffness_factor = get_stiffness_factor(experiment_id, self[WELL_INDEX_UUID])
-                else:
-                    # calibration recordings do not have an associated barcode or post stiffness since they
-                    # are creatd when a plate is not even on the istrument, so just set the stiffness factor to 1
-                    self.stiffness_factor = CALIBRATION_STIFFNESS_FACTOR
+            self.tissue_sampling_period = (
+                sampling_period if sampling_period else self[TISSUE_SAMPLING_PERIOD_UUID]
+            )
 
-                # extract datetimes
-                for uuid_ in (
-                    UTC_BEGINNING_RECORDING_UUID,
-                    UTC_BEGINNING_DATA_ACQUISTION_UUID,
-                    UTC_FIRST_TISSUE_DATA_POINT_UUID,
-                ):
-                    self[uuid_] = self._extract_datetime(uuid_)
+            if self.stiffness_override:
+                self.stiffness_factor = stiffness_factor
+            elif not self.get(IS_CALIBRATION_FILE_UUID, False):
+                # earlier versions of files do not have the IS_CALIBRATION_FILE_UUID in their metadata
+                experiment_id = get_experiment_id(self[PLATE_BARCODE_UUID])
+                self.stiffness_factor = get_stiffness_factor(experiment_id, self[WELL_NAME_UUID])
+            else:
+                # calibration recordings do not have an associated barcode or post stiffness since they
+                # are creatd when a plate is not even on the istrument, so just set the stiffness factor to 1
+                self.stiffness_factor = CALIBRATION_STIFFNESS_FACTOR
 
-                self.tissue_sampling_period = (
-                    sampling_period if sampling_period else self[TISSUE_SAMPLING_PERIOD_UUID]
+            if self.version < VersionInfo.parse("1.0.0"):  # type: ignore
+                # Ref data not yet added to Beta 2 files
+                self[UTC_FIRST_REF_DATA_POINT_UUID] = self._extract_datetime(UTC_FIRST_REF_DATA_POINT_UUID)
+                # setup noise filter, should probably add beta 2 file support for this
+                self.noise_filter_uuid = (
+                    TSP_TO_DEFAULT_FILTER_UUID[self.tissue_sampling_period] if self.is_magnetic_data else None
+                )
+                self.filter_coefficients = (
+                    create_filter(self.noise_filter_uuid, self.tissue_sampling_period)
+                    if self.noise_filter_uuid
+                    else None
                 )
 
-                if self.version < VersionInfo.parse("1.0.0"):
-                    # Ref data not yet added to Beta 2 files
-                    self[UTC_FIRST_REF_DATA_POINT_UUID] = self._extract_datetime(
-                        UTC_FIRST_REF_DATA_POINT_UUID
-                    )
-                    # setup noise filter, should probably add beta 2 file support for this
-                    self.noise_filter_uuid = (
-                        TSP_TO_DEFAULT_FILTER_UUID[self.tissue_sampling_period]
-                        if self.is_magnetic_data
-                        else None
-                    )
-                    self.filter_coefficients = (
-                        create_filter(self.noise_filter_uuid, self.tissue_sampling_period)
-                        if self.noise_filter_uuid
-                        else None
-                    )
+                self._load_magnetic_data()
+            else:
+                if self.has_inverted_post_magnet:
+                    raise ValueError("V1 algorithm will automatically fix this issue")
 
-                    # load sensor data. This is only possible to do here for Beta 1 data files
-                    self[TISSUE_SENSOR_READINGS] = self._load_reading(h5_file, TISSUE_SENSOR_READINGS)
-                    self[REFERENCE_SENSOR_READINGS] = self._load_reading(h5_file, REFERENCE_SENSOR_READINGS)
-                    self._load_magnetic_data()
-                else:
-                    if self.has_inverted_post_magnet:
-                        raise ValueError("V1 algorithm will automatically fix this issue")
+                self.noise_filter_uuid = None
+                self.filter_coefficients = None
+                self.is_magnetic_data = False
 
-                    self.noise_filter_uuid = None
-                    self.filter_coefficients = None
-                    self.is_magnetic_data = False
+            # extract datetime metadata
+            for uuid_ in (
+                UTC_BEGINNING_RECORDING_UUID,
+                UTC_BEGINNING_DATA_ACQUISTION_UUID,
+                UTC_FIRST_TISSUE_DATA_POINT_UUID,
+            ):
+                self[uuid_] = self._extract_datetime(uuid_)
 
-                    for dataset in (
-                        TIME_INDICES,
-                        TIME_OFFSETS,
-                        TISSUE_SENSOR_READINGS,
-                        REFERENCE_SENSOR_READINGS,
-                        STIMULATION_READINGS,
-                    ):
-                        self[dataset] = h5_file[dataset][:]
+            # format platemap metadata
+            for uuid_ in (PLATEMAP_NAME_UUID, PLATEMAP_LABEL_UUID):
+                val = self.get(uuid_, NOT_APPLICABLE_LABEL)
+                if val == str(NOT_APPLICABLE_H5_METADATA):
+                    val = NOT_APPLICABLE_LABEL
+                self[uuid_] = val
 
         elif file_path.endswith(".xlsx"):
             self._excel_sheet = _get_single_sheet(file_path)
@@ -171,19 +160,49 @@ class WellFile:
             self.is_force_data = (
                 "y" in str(_get_excel_metadata_value(self._excel_sheet, TWITCHES_POINT_UP_UUID)).lower()
             )
+            self.stiffness_factor = None
 
-            self.noise_filter_uuid = (
-                TSP_TO_DEFAULT_FILTER_UUID[self.tissue_sampling_period] if self.is_magnetic_data else None
-            )
-            self.filter_coefficients = (
-                create_filter(self.noise_filter_uuid, self.tissue_sampling_period)
-                if self.noise_filter_uuid
-                else None
-            )
+            for uuid_ in (PLATEMAP_NAME_UUID, PLATEMAP_LABEL_UUID):
+                self[uuid_] = NOT_APPLICABLE_LABEL
+            # skip all other transforms
+            self.force = self[TISSUE_SENSOR_READINGS].copy()
+            # timepoints still need to be in µs
+            self.force[0] *= MICRO_TO_BASE_CONVERSION
 
-            self.stiffness_factor = stiffness_factor if stiffness_factor else CARDIAC_STIFFNESS_FACTOR
+    def _load_data_from_h5_file(self, file_path: str) -> None:
+        with h5py.File(file_path, "r") as h5_file:
+            self.file_name = os.path.basename(h5_file.filename)
+            self.attrs = {attr: h5_file.attrs[attr] for attr in list(h5_file.attrs)}
+            self.version = self[FILE_FORMAT_VERSION_METADATA_KEY]
 
-            self._load_magnetic_data()
+            if self.version < VersionInfo.parse("1.0.0"):
+                # load sensor data. This is only possible to do here for Beta 1 data files
+                self[TISSUE_SENSOR_READINGS] = self._load_reading(h5_file, TISSUE_SENSOR_READINGS)
+                self[REFERENCE_SENSOR_READINGS] = self._load_reading(h5_file, REFERENCE_SENSOR_READINGS)
+            else:
+                for dataset in (
+                    TIME_INDICES,
+                    TIME_OFFSETS,
+                    TISSUE_SENSOR_READINGS,
+                    REFERENCE_SENSOR_READINGS,
+                    STIMULATION_READINGS,
+                ):
+                    self[dataset] = h5_file[dataset][:]
+
+    def _load_reading(self, h5_file, reading_type: str) -> NDArray[(Any, Any), int]:
+        sampling_period = self[
+            REF_SAMPLING_PERIOD_UUID
+            if reading_type == REFERENCE_SENSOR_READINGS
+            else TISSUE_SAMPLING_PERIOD_UUID
+        ]
+        time_step = int(sampling_period / MICROSECONDS_PER_CENTIMILLISECOND)
+
+        tissue_contraction_amplitudes = h5_file[reading_type][:]
+        num_data_points = tissue_contraction_amplitudes.shape[-1]
+
+        timepoints = np.arange(num_data_points) * time_step
+
+        return np.array([timepoints, tissue_contraction_amplitudes], dtype=np.int32)
 
     def _load_magnetic_data(self):
         adj_raw_tissue_reading = self[TISSUE_SENSOR_READINGS].copy()
@@ -304,21 +323,6 @@ class WellFile:
             tzinfo=datetime.timezone.utc
         )
 
-    def _load_reading(self, h5_file, reading_type: str) -> NDArray[(Any, Any), int]:
-        sampling_period = self[
-            REF_SAMPLING_PERIOD_UUID
-            if reading_type == REFERENCE_SENSOR_READINGS
-            else TISSUE_SAMPLING_PERIOD_UUID
-        ]
-        time_step = int(sampling_period / MICROSECONDS_PER_CENTIMILLISECOND)
-
-        tissue_contraction_amplitudes = h5_file[reading_type][:]
-        num_data_points = tissue_contraction_amplitudes.shape[-1]
-
-        timepoints = np.arange(num_data_points) * time_step
-
-        return np.array([timepoints, tissue_contraction_amplitudes], dtype=np.int32)
-
 
 class PlateRecording:
     def __init__(
@@ -330,11 +334,14 @@ class PlateRecording:
         # TODO unit test the stiffness factor (auto and override), inverted_post_magnet_wells
         stiffness_factor: Optional[int] = None,
         inverted_post_magnet_wells: Optional[List[str]] = None,
+        well_groups: Optional[Dict[str, List[str]]] = None,
     ):
         self.path = path
         self.wells = []
         self._iter = 0
+        # these may get overwritten later
         self.is_optical_recording = False
+        self.contains_stim_data = False
 
         # Tanner (11/16/22): due to the needs of the scientists for the full analysis,
         # these params should only be used in the recording snapshot.
@@ -352,14 +359,15 @@ class PlateRecording:
             with tempfile.TemporaryDirectory() as tmpdir:
                 zf = zipfile.ZipFile(path)
                 zf.extractall(path=tmpdir)
-                self.wells, calibration_recordings = load_files(
-                    tmpdir, stiffness_factor, inverted_post_magnet_wells
-                )
+
+                if glob.glob(os.path.join(tmpdir, "**", "*.h5"), recursive=True):
+                    self.wells, calibration_recordings = load_files(
+                        tmpdir, stiffness_factor, inverted_post_magnet_wells
+                    )
+                elif xlsx_files := glob.glob(os.path.join(tmpdir, "**", "*.xlsx"), recursive=True):
+                    self._load_optical_well_files(xlsx_files, stiffness_factor)
         elif self.path.endswith(".xlsx"):  # optical file
-            self.is_optical_recording = True
-            well_file = WellFile(self.path, stiffness_factor=stiffness_factor)
-            self.wells = [None] * (well_file[WELL_INDEX_UUID] + 1)
-            self.wells[well_file[WELL_INDEX_UUID]] = well_file
+            self._load_optical_well_files([self.path], stiffness_factor)
         else:  # .h5 files
             self.wells, calibration_recordings = load_files(
                 self.path, stiffness_factor, inverted_post_magnet_wells
@@ -369,6 +377,33 @@ class PlateRecording:
         if not any(self.wells):
             raise NoRecordingFilesLoadedError()
 
+        if len(self.wells) > len(set(w[WELL_NAME_UUID] for w in self.wells)):
+            raise DuplicateWellsFoundError()
+
+        # ensure wells are in correct order A1, B1,.., A2, B2,...
+        self.wells.sort(key=lambda w: (int(w[WELL_NAME_UUID][1:]), w[WELL_NAME_UUID][0]))
+
+        # set up platemap info
+        first_avaliable_well = next(iter(self))
+        self.platemap_name = first_avaliable_well[PLATEMAP_NAME_UUID]
+        platemap_labels = defaultdict(list)
+
+        for well_file in self:
+            if well_groups is None:
+                label = well_file[PLATEMAP_LABEL_UUID]
+                # only add to platemap_labels if label has been assigned
+                if label != NOT_APPLICABLE_LABEL:
+                    platemap_labels[label].append(well_file[WELL_NAME_UUID])
+            else:
+                # default all labels to NA first
+                well_file[PLATEMAP_LABEL_UUID] = NOT_APPLICABLE_LABEL
+                for label, well_names in well_groups.items():
+                    if well_file[WELL_NAME_UUID] in well_names:
+                        well_file[PLATEMAP_LABEL_UUID] = label
+                        platemap_labels[label].append(well_file[WELL_NAME_UUID])
+
+        self.platemap_labels = dict(platemap_labels)
+
         # currently file versions 1.0.0 and above must have all their data processed together
         if not self.is_optical_recording and self.wells[0].version >= VersionInfo.parse("1.0.0"):
             if self._created_from_dataframe:
@@ -381,6 +416,8 @@ class PlateRecording:
                 ):
                     self._process_stim_data()
 
+            self.contains_stim_data = any(wf.stim_sessions for wf in self)
+
     def _process_plate_data(self, calibration_recordings):
         if not all(isinstance(well_file, WellFile) for well_file in self.wells) or len(self.wells) != 24:
             raise NotImplementedError("All 24 wells must have a recording file present")
@@ -412,11 +449,15 @@ class PlateRecording:
         # create baseline data array
         baseline_data_mt = np.mean(baseline_data_mt[:, -BASELINE_MEAN_NUM_DATA_POINTS:], axis=1)
 
-        # pass data into magnet finding alg
-        log.info("Estimating magnet positions")
-        estimated_magnet_positions = find_magnet_positions(
-            plate_data_array_mt, baseline_data_mt, initial_magnet_finding_params
-        )
+        try:
+            # pass data into magnet finding alg
+            log.info("Estimating magnet positions")
+            estimated_magnet_positions = find_magnet_positions(
+                plate_data_array_mt, baseline_data_mt, initial_magnet_finding_params
+            )
+        except UnableToConvergeError:
+            log.exception("Unable to converge due to bad quality of data")
+            raise
 
         flip_data = self.wells[0].version >= VersionInfo.parse("1.1.0")
 
@@ -457,18 +498,17 @@ class PlateRecording:
                     stim_protocol["subprotocols"], wf[STIMULATION_READINGS], start_time_us, end_time_us
                 )
             except SubprotocolFormatIncompatibleWithInterpolationError:
-                log.info("Subprotocol format not supported by intperpolation")
+                log.exception("Subprotocol format not supported by interpolation")
                 return
 
-            charge_conversion_factor = (
-                MILLI_TO_BASE_CONVERSION if stim_protocol["stimulation_type"] == "C" else 1
-            )
+            is_voltage = stim_protocol["stimulation_type"] == "V"
+            charge_conversion_factor = 1 if is_voltage else MILLI_TO_BASE_CONVERSION
 
             for waveform in stim_sessions_waveforms:
                 if not waveform.shape[-1]:
                     continue
                 waveform[0] -= wf[TIME_INDICES][0]
-                waveform[1] //= charge_conversion_factor
+                waveform[1] /= charge_conversion_factor
                 wf.stim_sessions.append(waveform)
 
     def _load_dataframe(self, df: pd.DataFrame) -> None:
@@ -488,6 +528,7 @@ class PlateRecording:
             well_name = wf[WELL_NAME_UUID]
             raw_force_amplitudes = df[f"{well_name}__raw"]
             wf.force = np.vstack((force_timepoints, raw_force_amplitudes)).astype(np.float64)
+            wf.force = wf.force[:, ~np.isnan(raw_force_amplitudes)]
 
             if stim_timepoints is None:
                 continue
@@ -499,41 +540,25 @@ class PlateRecording:
                 stim_session = stim_session_raw[:, ~np.isnan(stim_session_raw[1])].astype(int)
                 wf.stim_sessions.append(stim_session)
 
-    def write_time_force_csv(self, output_dir: str):
-        # get recording name
-        recording_name = os.path.splitext(os.path.basename(self.path))[0]
-        output_path = os.path.join(output_dir, f"{recording_name}.csv")
+    def _load_optical_well_files(self, file_paths: List[str], stiffness_factor: Union[int, None]):
+        self.is_optical_recording = True
 
-        # set indexes to time points
-        truncated_time_sec = [
-            truncate_float(ms / MICRO_TO_BASE_CONVERSION, 2) for ms in self.wells[0].force[0]
-        ]
+        for xlsx_path in file_paths:
+            # check if xlsx is correct format and not pulse3d output file
+            if _get_num_of_sheets(xlsx_path) > 1:
+                raise IncorrectOpticalFileFormatError(
+                    f"Incorrect number of sheets found for file {os.path.basename(xlsx_path)}"
+                )
 
-        force_data = dict({"Time (s)": pd.Series(truncated_time_sec)})
+            well_file = WellFile(xlsx_path, stiffness_factor=stiffness_factor)
+            self.wells.append(well_file)
 
-        for well in self:
-            well_name = well.get(WELL_NAME_UUID, None)
-            force_data[well_name] = pd.Series(well.force[1])
-
-        time_recording_df = pd.DataFrame(force_data)
-        time_recording_df.to_csv(output_path, index=False)
-
-        return time_recording_df, output_path
-
-    def to_dataframe(self) -> pd.DataFrame:
+    def to_dataframe(self, include_stim_data=True) -> pd.DataFrame:
         """Creates DataFrame from PlateRecording with all the data
-        interpolated, normalized, and scaled. The returned dataframe contains
-        one column for time in ms and one column for each well.
+        interpolated, normalized, and scaled.
 
-        The dataframe returned by this method can be used in calls to peak_detector by selecting the
-        'time' column and the well column that peak detection should be run on after transposing the
-        data, e.g.
-
-        >>> df = to_dataframe()
-        >>> peak_detector(df[['time', 'A1']].transpose().values)
-
-        The dataframe needs to be transposed before converting to NDArray because peak_detector
-        wants an ndarray of shape (2, N) while dataframe[['time', 'A1']] is in shape (N,2)
+        The returned dataframe contains one column for time in ms and
+        one column for each well.
         """
         # get first valid well and set interpolation period. Creating new iter to be safe
         first_well = next(iter(self))
@@ -542,8 +567,9 @@ class PlateRecording:
         if self._created_from_dataframe:
             raise NotImplementedError("Cannot export a DF if created from a DF. Just use the original")
 
-        min_time = min([wf.force[0][0] for wf in self])
-        max_time = max([wf.force[0][-1] for wf in self])
+        min_time = min([wf.force[0, 0] for wf in self])
+        max_time = max([wf.force[0, -1] for wf in self])
+
         interp_period = (
             first_well[INTERPOLATION_VALUE_UUID] if self.is_optical_recording else INTERPOLATED_DATA_PERIOD_US
         )
@@ -551,22 +577,34 @@ class PlateRecording:
 
         data = {"Time (s)": pd.Series(interp_timepoints)}
 
-        output_stim_data = first_well.version >= VersionInfo.parse("1.0.0")
+        # only attempt to output stim data if the file supports it and the caller requests it
+        attempt_to_output_stim_data = (
+            not self.is_optical_recording
+            and first_well.version >= VersionInfo.parse(MIN_FILE_VERSION_FOR_STIM_INTERPOLATION)
+            and include_stim_data
+        )
 
         # add stim timepoints
-        if output_stim_data:
+        aggregate_stim_timepoints_us = None
+        if attempt_to_output_stim_data:
             aggregate_stim_timepoints_us = aggregate_timepoints(
                 [session_data[0] for wf in self for session_data in wf.stim_sessions]
             )
             aggregate_stim_timepoints_us_for_plotting = np.repeat(aggregate_stim_timepoints_us, 2)
+
+        # only outputting stim data if an attempt to output stim data was made and the file actually has stim data in it
+        is_outputting_stim_data = (
+            aggregate_stim_timepoints_us is not None and aggregate_stim_timepoints_us.any()
+        )
+        if is_outputting_stim_data:
             data["Stim Time (µs)"] = pd.Series(aggregate_stim_timepoints_us_for_plotting)
 
         # iterating over self.wells instead of using __iter__ so well_idx is preserved
-        for well_idx, wf in enumerate(self.wells):
+        for wf in self.wells:
             if not wf:
                 continue
 
-            well_name = wf.get(WELL_NAME_UUID, TWENTY_FOUR_WELL_PLATE.get_well_name_from_well_index(well_idx))
+            well_name = wf.get(WELL_NAME_UUID)
 
             # add raw force data
             data[f"{well_name}__raw"] = pd.Series(wf.force[1, :])
@@ -585,7 +623,7 @@ class PlateRecording:
             data[well_name] = pd.Series(interp_force_newtons_normalized)
 
             # add stim data
-            if output_stim_data:
+            if is_outputting_stim_data:
                 for i, session_data in enumerate(wf.stim_sessions):
                     data[f"{well_name}__stim_{i}"] = pd.Series(
                         realign_interpolated_stim_data(
@@ -594,29 +632,21 @@ class PlateRecording:
                     )
 
         df = pd.DataFrame(data)
-        if not output_stim_data:
+        if not is_outputting_stim_data:
             df.dropna(inplace=True)
 
         return df
 
     @staticmethod
-    def from_dataframe(path, df: pd.DataFrame):
+    def from_dataframe(path, **kwargs):
         # only allowed for one recording at a time assuming a user would only ever pass a dataframe to one recording
         log.info(f"Loading recording from file {os.path.basename(path)}")
-        yield PlateRecording(path, recording_df=df)
+        yield PlateRecording(path, **kwargs)
 
     @staticmethod
     def from_directory(path, **kwargs):
         # multi zip files
         for zf in glob.glob(os.path.join(path, "*.zip"), recursive=True):
-            zip_file = zipfile.ZipFile(zf)
-            # check if this is a zip of optical files
-            if ".xlsx" in zip_file.namelist()[0]:
-                with tempfile.TemporaryDirectory() as tmpdir:
-                    zip_file.extractall(path=tmpdir)
-                    for optical_file in os.scandir(tmpdir):
-                        yield PlateRecording(optical_file.path, **kwargs)
-                return
             log.info(f"Loading recording from file {zf}")
             yield PlateRecording(zf, **kwargs)
 
@@ -662,7 +692,7 @@ def load_files(
 
     for f in recording_files:
         log.info(f"Loading data from {os.path.basename(f)}")
-        well_name = os.path.splitext(f)[0].split("__")[1]
+        well_name = get_well_name_from_h5(f)
         well_file = WellFile(
             f,
             stiffness_factor=stiffness_factor,
@@ -678,16 +708,6 @@ def load_files(
     return tissue_well_files, baseline_well_files
 
 
-def _find_start_index(from_start: int, old_data: NDArray[(1, Any), int]) -> int:
-    start_index, time_from_start = 0, 0
-
-    while start_index + 1 < len(old_data) and from_start >= time_from_start:
-        time_from_start = old_data[start_index + 1] - old_data[0]
-        start_index += 1
-
-    return start_index - 1  # loop iterates 1 past the desired index, so subtract 1
-
-
 def _get_col_as_array(sheet: Worksheet, zero_based_row: int, zero_based_col: int) -> NDArray[(2, Any), float]:
     col_array = []
     result = _get_cell_value(sheet, zero_based_row, zero_based_col)
@@ -704,6 +724,11 @@ def _get_single_sheet(file_name: str) -> Any:
     return work_book[work_book.sheetnames[0]]
 
 
+def _get_num_of_sheets(file_path: str) -> Any:
+    work_book = load_workbook(file_path)
+    return len(work_book.sheetnames)
+
+
 def _get_cell_value(sheet: Worksheet, zero_based_row: int, zero_based_col: int) -> Optional[str]:
     result = sheet.cell(row=zero_based_row + 1, column=zero_based_col + 1).value
     if result is None:
@@ -741,12 +766,10 @@ def _load_optical_file_attrs(sheet: Worksheet):
 
     begin_recording = _get_excel_metadata_value(sheet, UTC_BEGINNING_RECORDING_UUID)
     begin_recording = datetime.datetime.strptime(begin_recording, "%Y-%m-%d %H:%M:%S")  # type: ignore
-
-    twenty_four_well = LabwareDefinition(row_count=4, column_count=6)
     well_name = _get_excel_metadata_value(sheet, WELL_NAME_UUID)
 
     attrs = {
-        FILE_FORMAT_VERSION_METADATA_KEY: "N/A",
+        FILE_FORMAT_VERSION_METADATA_KEY: NOT_APPLICABLE_LABEL,
         TISSUE_SENSOR_READINGS: raw_tissue_reading,
         REFERENCE_SENSOR_READINGS: np.zeros(raw_tissue_reading.shape),
         str(INTERPOLATION_VALUE_UUID): interpolation_value,
@@ -755,7 +778,6 @@ def _load_optical_file_attrs(sheet: Worksheet):
         str(MANTARRAY_SERIAL_NUMBER_UUID): _get_excel_metadata_value(sheet, MANTARRAY_SERIAL_NUMBER_UUID),
         str(PLATE_BARCODE_UUID): _get_excel_metadata_value(sheet, PLATE_BARCODE_UUID),
         str(WELL_NAME_UUID): well_name,
-        str(WELL_INDEX_UUID): twenty_four_well.get_well_index_from_well_name(well_name),
     }
 
     return attrs
